"""
MessageWindow.py
by Jason Conti
February 19, 2010

Popup window for viewing notifications.
"""

import gnomeapplet
import gtk
import pango

class MessageWindow(gtk.Window):
  """Shows the log of notification messages."""
  def __init__(self, parent, limit_messages = 5):
    gtk.Window.__init__(self)

    self._parent = parent
    self._limit_messages = limit_messages

    self.set_type_hint(gtk.gdk.WINDOW_TYPE_HINT_DOCK)
    self.set_decorated(False)
    self.set_resizable(False)
    self.stick()
    self.set_title("Message Window")
    self.set_border_width(5)
    self.set_size_request(400, -1)

    self._vbox = gtk.VBox(True, 5)
    self.add(self._vbox)

    self._empty_label = gtk.Label("No messages")
    self._vbox.pack_start(self._empty_label)

    self._messages = []

  def add_message(self, message):
    """Adds a message to the message window."""
    widget = MessageWidget(message)
    self._messages.append(widget)
    self._vbox.pack_start(widget)

    if self._empty_label != None:
      self._vbox.remove(self._empty_label)
      self._empty_label.destroy()
      self._empty_label = None

    if len(self._messages) > self._limit_messages:
      message = self._messages.pop(0)
      self._vbox.remove(message)
      message.destroy()

  def align_with_parent(self):
    """Adapted from position_calendar_popup in the GNOME clock applet."""
    # Get the size of the applet button and the size of this window
    x, y = self._parent.get_origin()
    w, h = self.size_request()
    button_w, button_h = self._parent.get_size()

    screen = self.get_screen()
    monitor = None

    # Figure out which monitor we are on
    for i in range(screen.get_n_monitors()):
      m = screen.get_monitor_geometry(i)

      if x >= m.x and x <= m.x + m.width and y >= m.y and y <= m.y + m.height:
        monitor = m
        break

    # Probably xinerama
    if monitor == None:
      monitor = gtk.gdk.Rectangle(0, 0, screen.get_width(), screen.get_height())

    orient = self._parent.get_orient()

    # Figure out the x, y coordinates of the window and the gravity depending
    # on the orientation of the panel
    if orient == gnomeapplet.ORIENT_RIGHT:  # Panel on left
      x += button_w
      if (y + h) > monitor.y + monitor.height:
        y -= (y + h) - (monitor.y + monitor.height)

      if (y + h) > (monitor.height / 2):
        gravity = gtk.gdk.GRAVITY_SOUTH_WEST
      else:
        gravity = gtk.gdk.GRAVITY_NORTH_WEST

    elif orient == gnomeapplet.ORIENT_LEFT: # Panel on right
      x -= w
      if (y + h) > monitor.y + monitor.height:
        y -= (y + h) - (monitor.y + monitor.height)

      if (y + h) > (monitor.height / 2):
        gravity = gtk.gdk.GRAVITY_SOUTH_EAST
      else:
        gravity = gtk.gdk.GRAVITY_NORTH_EAST

    elif orient == gnomeapplet.ORIENT_DOWN: # Panel on top
      y += button_h
      if (x + w) > monitor.x + monitor.width:
        x -= (x + w) - (monitor.x + monitor.width)

      gravity = gtk.gdk.GRAVITY_NORTH_WEST

    else: # Panel on bottom
      y -= h
      if (x + w) > monitor.x + monitor.width:
        x -= (x + w) - (monitor.x + monitor.width)

      gravity = gtk.gdk.GRAVITY_SOUTH_WEST

    self.move(x, y)
    self.set_gravity(gravity)

class MessageWidget(gtk.Frame):
  """Widget to display a Notifications.Message."""
  def __init__(self, message):
    gtk.Frame.__init__(self)

    self._message = message

    self._vbox = gtk.VBox(False)
    self.add(self._vbox)

    self._hbox = gtk.HBox(False)
    self._vbox.pack_start(self._hbox, True, True)

    icon_pixbuf = message.get_icon_pixbuf()
    if icon_pixbuf != None:
      self._icon_image = gtk.image_new_from_pixbuf(icon_pixbuf)
      self._icon_image.set_alignment(0, 0)
      self._icon_image.set_padding(5, 5)
      self._hbox.pack_start(self._icon_image, False, False)

    self._text_vbox = gtk.VBox(False)
    self._hbox.pack_start(self._text_vbox, True, True)

    self._title_label = gtk.Label(message.summary)
    self._title_label.set_alignment(0, 0)
    self._title_label.set_padding(5, 5)
    attr = pango.AttrList()
    attr.insert(pango.AttrWeight(pango.WEIGHT_BOLD, 0, -1))
    self._title_label.set_attributes(attr)
    self._text_vbox.pack_start(self._title_label, False, False)

    self._body_label = MessageBody(message.body)
    self._body_label.set_alignment(0, 0)
    self._body_label.set_padding(5, 5)
    self._body_label.set_line_wrap(True)
    self._text_vbox.pack_start(self._body_label, True, True)

    self._timestamp_label = gtk.Label(message.formatted_timestamp() + " from " + message.app_name)
    self._timestamp_label.set_padding(5, 5)
    attr = pango.AttrList()
    attr.insert(pango.AttrScale(pango.SCALE_SMALL, 0, -1))
    self._timestamp_label.set_attributes(attr)
    self._vbox.pack_start(self._timestamp_label, False, False)

class MessageBody(gtk.Label):
  """Implements a workaround from:
    http://www.16software.com/blog/dynamic-label-wrapping-in-gtk
    that fixes the label overflow problems I have been having
    with a long message body."""
  def __init__(self, text):
    gtk.Label.__init__(self, text)

    self.set_line_wrap(True)
    self.connect("size-allocate", self.on_size_allocate)

  def on_size_allocate(self, widget, rect):
    self.set_size_request(rect.width - 10, -1)
