"""
Monitor.py
by Jason Conti
February 19, 2010

Monitors DBUS for org.freedesktop.Notifications.Notify messages, parses them,
and notifies listeners when they arrive.
"""

import dbus
import glib
import gobject
import gtk
import logging
import time

from dbus.mainloop.glib import DBusGMainLoop

class Message(gobject.GObject):
  """Parses a DBUS message in the Notify format specified at:
  http://www.galago-project.org/specs/notification/0.9/index.html"""
  def __init__(self, dbus_message, timestamp):
    gobject.GObject.__init__(self)

    self.timestamp = timestamp

    args = dbus_message.get_args_list()

    self.app_name = str(args[0])
    self.replaces_id = args[1]
    self.app_icon = args[2]
    self.summary = str(args[3])
    self.body = str(args[4])
    self.actions = args[5]
    self.hints = args[6]
    self.expire_timeout = args[7]

    self.log_message()

  def get_icon_pixbuf(self):
    """Loads the icon into a pixbuf. Adapted from the load_icon code in
    bubble.c of notify-osd."""
    icon_name = str(self.app_icon)
    
    # Try to load the pixbuf from a file
    if icon_name.startswith("file://") or icon_name.startswith("/"):
      try:
        return gtk.gdk.pixbuf_new_from_file_at_size(icon_name, 48, 48)
      except:
        return None

    # Try to load the pixbuf from the current icon theme
    try:
      theme = gtk.icon_theme_get_default()
      return theme.load_icon(icon_name, 48, gtk.ICON_LOOKUP_FORCE_SVG)
    except:
      return None

  def formatted_timestamp(self):
    """Returned the timestamp in a different format."""
    return time.strftime("%B %d, %Y at %I:%M:%S %p", self.timestamp)

  def log_message(self):
    """Write debug info about a message."""
    logging.debug("-" * 50)
    logging.debug("Message created at: " + self.formatted_timestamp())
    logging.debug("app_name: " + repr(self.app_name))
    logging.debug("replaces_id: " + repr(self.replaces_id))
    logging.debug("app_icon: " + repr(self.app_icon))
    logging.debug("summary: " + repr(self.summary))
    logging.debug("body: " + repr(self.body))
    logging.debug("actions: " + repr(self.actions))
    logging.debug("hints: " + repr(self.hints))
    logging.debug("expire_timeout: " + repr(self.expire_timeout))
    logging.debug("-" * 50)

class Monitor(gobject.GObject):
  """Monitors DBUS for org.freedesktop.Notifications.Notify messages, parses them,
  and notifies listeners when they arrive."""
  __gsignals__ = {
      "message-received": (gobject.SIGNAL_RUN_LAST, gobject.TYPE_NONE, [Message])
  }
  def __init__(self):
    gobject.GObject.__init__(self)

    self._match_string = "type='method_call',interface='org.freedesktop.Notifications',member='Notify'"

    DBusGMainLoop(set_as_default=True)
    self._bus = dbus.SessionBus()
    self._bus.add_match_string(self._match_string)
    self._bus.add_message_filter(self._message_filter)

  def close(self):
    self._bus.close()

  def _message_filter(self, connection, dbus_message):
    if dbus_message.get_member() == "Notify" and dbus_message.get_interface() == "org.freedesktop.Notifications":
      glib.idle_add(self.emit, "message-received", Message(dbus_message, time.localtime()))

def test_receiver(w, m):
  print "-"*50
  print "timestamp", m.formatted_timestamp()
  print "app_name", repr(m.app_name)
  print "replaces_id", repr(m.replaces_id)
  print "app_icon", repr(m.app_icon)
  print "summary", repr(m.summary)
  print "body", repr(m.body)
  print "actions", repr(m.actions)
  print "hints", repr(m.hints)
  print "expire_timeout", repr(m.expire_timeout)
  print "-"*50

def main():
  x = Monitor()
  x.connect("message-received", test_receiver)
  gtk.main()

if gtk.pygtk_version < (2, 8, 0):
  gobject.type_register(Message)
  gobject.type_register(Monitor)

if __name__ == '__main__':
  main()
